/* $Id: hdcs.c,v 1.2 2007/02/20 15:56:29 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>

#include "openqce.h"

void
hdcs_set4(struct quickcam *qce, byte reg1, byte reg2, byte reg3, byte reg4,
		byte val1, byte val2, byte val3, byte val4)
{
	byte i2cbuf[35];

	i2cbuf[0x00] = reg1 * 2;
	i2cbuf[0x01] = reg2 * 2;
	i2cbuf[0x02] = reg3 * 2;
	i2cbuf[0x03] = reg4 * 2;

	i2cbuf[0x10] = val1;
	i2cbuf[0x11] = val2;
	i2cbuf[0x12] = val3;
	i2cbuf[0x13] = val4;

	i2cbuf[0x20] = 0xAA;	// bus address
	i2cbuf[0x21] = 0x03;	// 4 registers
	i2cbuf[0x22] = 0x01;	// write

	stv_set(qce, 35, 0x0400, i2cbuf);
}

void
hdcs_set3(struct quickcam *qce, byte reg1, byte reg2, byte reg3,
		byte val1, byte val2, byte val3)
{
	byte i2cbuf[35];

	i2cbuf[0x00] = reg1 * 2;
	i2cbuf[0x01] = reg2 * 2;
	i2cbuf[0x02] = reg3 * 2;

	i2cbuf[0x10] = val1;
	i2cbuf[0x11] = val2;
	i2cbuf[0x12] = val3;

	i2cbuf[0x20] = 0xAA;	// bus address
	i2cbuf[0x21] = 0x02;	// 3 registers
	i2cbuf[0x22] = 0x01;	// write

	stv_set(qce, 35, 0x0400, i2cbuf);
}

void
hdcs_set1(struct quickcam *qce, byte reg, byte val)
{
	byte i2cbuf[35];

	i2cbuf[0x00] = reg * 2;

	i2cbuf[0x10] = val;

	i2cbuf[0x20] = 0xAA;	// bus address
	i2cbuf[0x21] = 0x00;	// 1 register
	i2cbuf[0x22] = 0x01;	// write

	stv_set(qce, 35, 0x0400, i2cbuf);
}

byte
hdcs_get(struct quickcam *qce, byte reg)
{
	byte i2cbuf[35];
	byte val;

	i2cbuf[0x00] = reg * 2;

	i2cbuf[0x20] = 0xAA;	// bus address
	i2cbuf[0x21] = 0x00;	// 1 register
	i2cbuf[0x22] = 0x03;	// read

	stv_set(qce, 35, 0x1400, i2cbuf);
	val = stv_get_byte(qce, STV_I2CRETURN);

	return (val);
}


void
hdcs_extra_dump(struct quickcam *qce)
{
	byte b;
	char dod_mode[4][32] = {
		"Driven with ADC_data (10bits).",
		"Rounded to 8 significant bits.",
		"Saturation mode 2.",
		"Saturation mode 1." };
	char bpf_frac[4][25] = {
		"Zero fractional portion.",
		"1/4", "1/2", "3/4" };

	b = hdcs_get(qce, HDCS_IMASK);
	printf("\nInterrupt Mask (IMASK)\n");
	printf("  Enabled......................... %u\n", !!(b & 1<<0));
	printf("  When row complete............... %u\n", !!(b & 1<<1));
	printf("  When frame complete............. %u\n", !!(b & 1<<2));
	printf("  When capture complete........... %u\n", !!(b & 1<<3));
	printf("  When interface error............ %u\n", !!(b & 1<<4));
	printf("  When shutter sync............... %u\n", !!(b & 1<<5));

	b = hdcs_get(qce, HDCS_PCTRL);
	printf("\nPad Control (PCTRL), signals\n");
	printf("  Row complete.................... %u\n", !!(b & 1<<0));
	printf("  Frame complete/Shutter sync..... %u\n", !!(b & 1<<1));
	printf("  Multifunction pin............... %u\n", !!(b & 1<<2));
	printf("  Capture complete................ %u\n", !!(b & 1<<3));
	printf("  Interrupt pin pull-up........... %u\n", !!(b & 1<<4));
	printf("  Level row status................ %u\n", !!(b & 1<<5));
	printf("  Level frame status.............. %u\n", !!(b & 1<<6));
	printf("  Level cap. status............... %u\n", !!(b & 1<<7));

	b = hdcs_get(qce, HDCS_PDRV);
	printf("\nPad Drive Control (PDRV), drive levels\n");
	printf("  Parallel data port.............. %u ns\n", ((b & 3) + 1) * 5);
	printf("  DRDY signal..................... %u ns\n", (((b & 12)>>2) + 1) * 5);
	printf("  nRow, tclk_nFramce, nIRQ_nCC.... %u ns\n", (((b & 48)>>4) + 1) * 5);
	printf("  Parallel data port drive level.. %u ns\n", (((b & 192)>>6) + 1) * 5);

	b = hdcs_get(qce, HDCS_ICTRL);
	printf("\nInterface Control (ICTRL)\n");
	printf("  Auto Address (AAD).............. %s\n", 
			(b & 1 << 0) ? "Disabled" : "Enabled");
	printf("  Device Address (DAD)............ %s\n",
			(b & 1 << 1) ? "Disabled" : "Enabled");
	printf("  Data Output (DOD)............... %s\n", 
			dod_mode[(b & 12) >> 2]);
	printf("  Delay data output relative to... %s of the system clock.\n",
			(b & 1 << 4) ? "Falling edge" : "Rising edge");
	printf("  Data Setup cycle count.......... %u clocks before DRDY assertion.\n",
			((b & 48) >> 5));
	printf("  Horizontal average (HAVG)....... %s\n",
			(b & 1 << 7) ? "Enabled" : "Disabled");

	b = hdcs_get(qce, HDCS_ITMG);
	printf("\nInterface Timing Control (ITMG)\n");
	printf("  Data Ready Pulse Count (RPC).... %u clocks.\n", (b & 7) + 1);
	printf("  Data Hold Count (DHC)........... %u clocks.\n", (b & 24)>>3);
	printf("  DRDY signal Polarity (DPS)...... DRDY is active %s.\n",
			(b & 32) ? "low" : "high");

	b = hdcs_get(qce, HDCS_BFRAC);
	printf("\nBaud Rate Fraction................ %s\n", bpf_frac[(b & 3)]);

	b = hdcs_get(qce, HDCS_BRATE);
	printf("Baud Rate (integer portion)....... %u\n", b);
	printf("Baud Rate (total)................. %s\n", "CLK_FREQ * [ 16 * (BPI + 1) + (4 * BPF)]");

	printf("\nTiming Control (TCTRL)\n");
	b = hdcs_get(qce, HDCS_TCTRL);
	printf("  PGA Sample duration (PSMP)...... %u clock cycles\n", 
			(b & 0x1F));
	printf("  adcStart Signal Duration........ %u clock cycles\n", 
			(b & 0x60) + 2);
	printf("  ADC Sample Duration............. %u clock cycles\n", 
			(b & 0x60) + 1);


	b = hdcs_get(qce, HDCS_ADCCTRL);
	printf("\nADC Conversion Resolution (ARES).. %u bits\n", (b & 0xF));

	printf("\nGeometry\n");
	b = hdcs_get(qce, HDCS_FWROW);
	printf("  First Row Address............... %u\n", (b & 0x7F));
	b = hdcs_get(qce, HDCS_FWCOL);
	printf("  First Column Address............ %u\n", b);
	b = hdcs_get(qce, HDCS_LWROW);
	printf("  Last Row Address................ %u\n", (b & 0x7F));
	b = hdcs_get(qce, HDCS_LWCOL);
	printf("  Last Column Address............. %u\n", b);


	printf("\nPGA Gain\n");
	b = hdcs_get(qce, HDCS_ERECPGA);
	printf("  Even Row, Even Column (green1).. %02X\n", b);
	b = hdcs_get(qce, HDCS_EROCPGA);
	printf("  Even Row, Odd Column (red)...... %02X\n", b);
	b = hdcs_get(qce, HDCS_ORECPGA);
	printf("  Odd Row, Even Column (blue)..... %02X\n", b);
	b = hdcs_get(qce, HDCS_OROCPGA);
	printf("  Odd Row, Odd Column (green1).... %02X\n", b);

	printf("\nExposure\n");
	b = hdcs_get(qce, HDCS_ROWEXPL);
	printf("  Row Exposure Low (ROWEXPL)...... %02X\n", b);
	b = hdcs_get(qce, HDCS_ROWEXPH);
	printf("  Row Exposure High (ROWEXPH)..... %02X\n", b);
	b = hdcs_get(qce, HDCS_SROWEXPL);
	printf("  Sub-Row Exposure Low (SROWEXPL). %02X\n", b);
	b = hdcs_get(qce, HDCS_SROWEXPH);
	printf("  Sub-Row Exposure High (SROWEXPH) %02X\n", b);
}

void
dump_hdcs(struct quickcam *qce, int verbose)
{
	byte b;
	char opmode[4][32] = {
		"Normal operation",
		"Shutter",
		"Accumulation",
		"Reserved" };

	printf("--------------------------------------------\n");
	b = hdcs_get(qce, HDCS_IDENT);
	printf(" HDCS Registers - %s, revision %s\n", 
			b & 8 ? "HDCS1000/1010" : "HDCS2000/2010", 
			b & 7 ? "Unknown" : "A");
	printf("--------------------------------------------\n");


	b = hdcs_get(qce, HDCS_STATUS);
	printf("\nStatus\n");
	printf("  Running......................... %u\n", !!(b & 1<<0));
	printf("  Row completed................... %u\n", !!(b & 1<<1));
	printf("  Frame completed................. %u\n", !!(b & 1<<2));
	printf("  Capture completed............... %u\n", !!(b & 1<<3));
	printf("  Exposure error.................. %u\n", !!(b & 1<<4));
	printf("  Interface error................. %u\n", !!(b & 1<<5));
	printf("  Shutter sync.................... %u\n", !!(b & 1<<6));

	b = hdcs_get(qce, HDCS_CONFIG);
	printf("\nConfiguration\n");
	printf("  Operation mode.................. %s\n", opmode[b & 3]);
	printf("  Stop when frame completed....... %u\n", !!(b & 1 << 2));
	printf("  Continuous frame capture........ %u\n", !!(b & 1 << 3));
	printf("  Column Sub-Sample............... %u\n", !!(b & 1 << 4));
	printf("  Row Sub-Sample.................. %u\n", !!(b & 1 << 5));

	if (verbose)
		hdcs_extra_dump(qce);
	/* TODO le reste */


}


