/* $Id: openqce.h,v 1.4 2007/02/21 18:12:13 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */

#define DEFAULT_FORMAT	4
#define DEFAULT_VSIZE	2


/* STV0600 Registers */
/* this one is write-only, anything >0 in windows makes it blink */
#define STV_REG001	0x0423
#define STV_REG002	0x0424
#define STV_I2CRETURN	0x1410
#define STV_STREAMING	0x1440
#define STV_SCANRATE	0x1443
#define STV_REG003	0x1444
#define STV_REG004	0x1500
#define STV_REG007	0x1501
#define STV_REG008	0x1502
#define STV_REG006	0x1503
#define STV_REG005	0x1504
#define STV_ISOSIZE_MAX	0x15C1
#define STV_CTRL_Y	0x15C3
#define STV_REG009	0x15C4
#define STV_CTRL_X	0x1680

/* HDCS registers */
/* addresses from the documentation (doubled in the code) */
#define HDCS_IDENT	0x00
#define HDCS_STATUS	0x01
#define HDCS_IMASK	0x02
#define HDCS_PCTRL	0x03
#define HDCS_PDRV	0x04
#define HDCS_ICTRL	0x05
#define HDCS_ITMG	0x06
#define HDCS_BFRAC	0x07
#define HDCS_BRATE	0x08
#define HDCS_ADCCTRL	0x09
#define HDCS_FWROW	0x0A
#define HDCS_FWCOL	0x0B
#define HDCS_LWROW	0x0C
#define HDCS_LWCOL	0x0D
#define HDCS_TCTRL	0x0E
#define HDCS_ERECPGA	0x0F
#define HDCS_EROCPGA	0x10
#define HDCS_ORECPGA	0x11
#define HDCS_OROCPGA	0x12
#define HDCS_GAIN_G1	0x0F
#define HDCS_GAIN_R	0x10
#define HDCS_GAIN_B	0x11
#define HDCS_GAIN_G2	0x12
#define HDCS_ROWEXPL	0x13
#define HDCS_ROWEXPH	0x14
#define HDCS_SROWEXPL	0x15
#define HDCS_SROWEXPH	0x16
#define HDCS_CONFIG	0x17
#define HDSC_CONTROL	0x18

#define COL_R		0
#define COL_G		1
#define COL_B		2


typedef unsigned char byte;

struct quickcam {
	byte		*rawbuf;
	byte		*rgbframe;
	byte		 isobuf[131072];
	char		 devname[11];
	unsigned int	 framecount;
	unsigned long	 pos;
	unsigned long	 resume;
	int		 ctrl_ep, iso_ep;
	int		 recording;
	unsigned int	 height, width;
	unsigned short	 format, mode;
	byte		 fwrow, fwcol, lwrow, lwcol;
};

/* prototypes for stv.c */
int stv_get(struct quickcam *, int size, int reg, char *buffer);
char stv_get_byte(struct quickcam *, int reg);
void stv_get_word(struct quickcam *, int reg, int *buf);
int stv_set(struct quickcam *, int size, int reg, unsigned char *buffer);
void stv_set_byte(struct quickcam *, int reg, char byte);
void stv_dump(struct quickcam *);
void stv_full_dump(struct quickcam *);

/* prototypes for hdcs.c */
void hdcs_set4(struct quickcam *, byte, byte, byte, byte, byte, byte, byte, byte);
void hdcs_set3(struct quickcam *, byte, byte, byte, byte, byte, byte);
void hdcs_set1(struct quickcam *, byte, byte);
byte hdcs_get(struct quickcam *, byte);
void dump_hdcs(struct quickcam *, int);

/* prototypes for quickcam.c */
struct quickcam *quickcam_create(void);
void quickcam_init(struct quickcam *);
void quickcam_capture_start(struct quickcam *);
void quickcam_capture_stop(struct quickcam *);
void quickcam_next_frame(struct quickcam *);
void quickcam_shutdown(struct quickcam *);

/* prototypes for filters.c */
void filter_dump_raw(struct quickcam *);
void filter_render_grey(struct quickcam *);
void filter_render_rgb(struct quickcam *);
void filter_render_2x2(struct quickcam *);
void filter_render_full2x2(struct quickcam *);


