/* $Id: quickcam.c,v 1.3 2007/02/21 18:12:13 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <dev/usb/usb.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <err.h>
#include <string.h>

#include "openqce.h"


struct quickcam *
quickcam_create()
{
	struct quickcam *qce;

	qce = malloc(sizeof(struct quickcam));
	qce->mode = DEFAULT_VSIZE;
	qce->format = DEFAULT_FORMAT;
	qce->pos = 0;
	qce->resume = NULL;
	qce->framecount = 0;
	qce->recording = 1;
	strlcpy(qce->devname, "/dev/ugen0", 11);

	return (qce);
}


void
quickcam_init(struct quickcam *qce)
{
	struct usb_alt_interface alt;
	char endpt[14];


	if (strncmp(qce->devname, "/dev/ugen", 9) != 0) {
		errx(-1, "Invalid device name (expecting \"/dev/ugenX\").");
	}

	/* control endpoint */
	snprintf(endpt, 14, "%s.00", qce->devname);
	if ((qce->ctrl_ep = open(endpt, O_WRONLY)) == -1) {
		errx(-1, "Unable to open control endpoint.");
	}

	/* alternate interface */
	alt.uai_interface_index = 0;
	alt.uai_alt_no = 1;
	if (ioctl(qce->ctrl_ep, USB_SET_ALTINTERFACE, &alt) < 0) {
		errx(-1, "Unable to set alternate interface.");
	}

	/* isochronous endpoint */
	//qce->iso_ep = open("dump", O_RDONLY);
	snprintf(endpt, 14, "%s.01", qce->devname);
	if ((qce->iso_ep = open(endpt, O_RDONLY)) == -1) {
		errx(-1, "Unable to open isochronous endpoint.");
	}

	/* take in consideration user resolution choice */
	switch (qce->mode) {
		case 0:
			qce->width = 376;
			qce->height = 312;
			qce->fwrow = 0;
			qce->fwcol = 0;
			qce->lwrow = 0x4D;
			qce->lwcol = 0x5D;
			break;
		case 1:
			qce->width = 360;
			qce->height = 296;
			qce->fwrow = 2;
			qce->fwcol = 2;
			qce->lwrow = 0x4B;
			qce->lwcol = 0x5B;
			break;
		case 2:
		default:
			qce->width = 352;
			qce->height = 288;
			qce->fwrow = 3;
			qce->fwcol = 3;
			qce->lwrow = 0x4A;
			qce->lwcol = 0x5A;
			break;
	}
	qce->rawbuf = malloc(qce->width * qce->height);
	qce->rgbframe = malloc(qce->width * qce->height * 3);


	/* stop streaming */
	stv_set_byte(qce, STV_STREAMING, 0);

	/* unknown */
	stv_set_byte(qce, STV_REG001, 0x05);

	/* read address 0 of I2C device 0xBA */
	;

	/* read STV_REG002 (returns 0x0A)*/
	;

	/* unknown */
	stv_set_byte(qce, STV_REG001, 0x04);

	/* read STV_REG003 *three* times (always returns 0x02) */
	;

	/* unknown */
	stv_set_byte(qce, STV_REG001, 0x05);

	/* read address 0 and 1 of I2C device 0x20,
	 * read STV_REG001 (returns 0x0a)
	 * -> those two ops, three times... */
	;

	/* unknown */
	stv_set_byte(qce, STV_REG001, 0x04);
	stv_set_byte(qce, STV_REG001, 0x04);

	/* ask for HDCS_STATUS */
	/* read STV_REG002 (returns 0x0a) */
	/* read STV_I2CRETURN for HDCS_STATUS answer */
	/* -> those three ops, two times... */

	/* set STV_REG004 to 0x1D */
	stv_set_byte(qce, STV_REG004, 0x1D);

	/* read STV_SCANRATE (returns 0) */
	/* set STV_SCANRATE at 1 */
	stv_set_byte(qce, STV_SCANRATE, 0x01);
	/* read STV_SCANRATE (returns 1) */
	/* set STV_SCANRATE at 0 */
	stv_set_byte(qce, STV_SCANRATE, 0x00);
	/* set STV_SCANRATE at 0 */
	stv_set_byte(qce, STV_SCANRATE, 0x00);

	/* set HDCS registers */
	hdcs_set4(qce, HDCS_PCTRL, HDCS_PDRV, HDCS_CONFIG, HDCS_ADCCTRL,
			0x63, 0x00, 0x08, 0x0A);

	/* set HDCS gain */
	hdcs_set4(qce, HDCS_GAIN_G1, HDCS_GAIN_R, HDCS_GAIN_B, HDCS_GAIN_G2,
			0x10, 0x10, 0x10, 0x10);

	/* set STV_REG005 to 0x07 */
	stv_set_byte(qce, STV_REG005, 0x07);

	/* set STV_REG006 to 0x45 */
	stv_set_byte(qce, STV_REG006, 0x95);

	/* read STV_REG004 (return 0x1D) */
	;

	/* set STV_REG004 to 0x1D */
	stv_set_byte(qce, STV_REG004, 0x1D);

	/* read USB vendor stuff.. */
	;

	/******* EVERYTHING ABOVE THIS LINE WAS BEFORE STARTING STREAMING,
	******** WEBCAM GET PLUGGED, THEN WE START WINDOWS CAPTURE. */

	/* set STV_REG001 to 0x04 */
	stv_set_byte(qce, STV_REG001, 0x04);

	/* set STV_REG004 to 0x1D */
	stv_set_byte(qce, STV_REG004, 0x1D);

	/* set HDCS registers */
	hdcs_set4(qce, HDCS_PCTRL, HDCS_PDRV, HDCS_CONFIG, HDCS_ADCCTRL,
			0x63, 0x00, 0x08, 0x0A);

	/* set STV_CTRL_Y 02 / 01*/
	stv_set_byte(qce, STV_CTRL_Y, 0x01);

	/* set STV_ISOSIZE_MAX to 0x034D */
	char maxiso[2] = { 0x4F, 0x03 };
	stv_set(qce, 2,   STV_ISOSIZE_MAX, maxiso);

	/* Setting the capture window */
	hdcs_set4(qce, HDCS_FWROW, HDCS_FWCOL, HDCS_LWROW, HDCS_LWCOL, 
			qce->fwrow, qce->fwcol, qce->lwrow, qce->lwcol);

	/* Setting STV_CTRL_X 0A/06 */
	stv_set_byte(qce, STV_CTRL_X, 0x0A);

	/* Set HDCS control stuff */
	//hdcs_set3(qce, HDCS_TCTRL, HDCS_ICTRL, HDCS_ITMG, 0x18, 0x00, 0x16);
	hdcs_set3(qce, HDCS_TCTRL, HDCS_ICTRL, HDCS_ITMG, 0x09, 0x20, 0x12);

	/* Set STV_SCANRATE to 0 */
	stv_set_byte(qce, STV_SCANRATE, 0x20);

	/* set exposure */
	//hdcs_set4(qce, HDCS_ROWEXPL, HDCS_ROWEXPH, HDCS_SROWEXPL, HDCS_SROWEXPH,
	//		0x92, 0x00, 0x80, 0x06);
	hdcs_set4(qce, HDCS_ROWEXPL, HDCS_ROWEXPH, HDCS_SROWEXPL, HDCS_SROWEXPH,
			0x80, 0x00, 0x00, 0x01);

	/* set STV_REG007 to 0xB5 */
	stv_set_byte(qce, STV_REG007, 0xB5);
	
	/* set STV_REG007 to 0xB5 */
	stv_set_byte(qce, STV_REG008, 0xA8);

	/* Set some more stuff in HDCS */
	hdcs_set4(qce, HDCS_PCTRL, HDCS_PDRV, HDCS_CONFIG, HDCS_ADCCTRL, 
			0x63, 0x00, 0x08, 0x0A);

	/***** HERE INIT SHOULD BE OVER ... THE NEXT OP SHOULD START THE
	****** STREAMING... */
}


void
quickcam_shutdown(struct quickcam *qce)
{

	if (close(qce->ctrl_ep) == -1) {
		errx(-1, "Error while closing control endpoint.");
	}

	if (close(qce->iso_ep) == -1) {
		errx(-1, "Error while closing isochronous endpoint.");
	}

	free(qce->rawbuf);
	free(qce);

	exit(0);
}


void
save_chunk(struct quickcam *qce, byte *data, int size)
{
	//fprintf(stderr, "save_chunk size=%u at pos=%lu\n", size, qce->pos);
	if (qce->pos + size > qce->width * qce->height) {
		warnx("Buffer overflow!");
		return;
	}
	memcpy(qce->rawbuf + qce->pos, data, size);
	qce->pos += size;
}


int
parse_iso(struct quickcam *qce, unsigned long size)
{
	unsigned long i;
	unsigned int csize;
	byte a, b, c, d;

	//fprintf(stderr,"parse_iso (%lu bytes)\n", size);

	qce->resume = 0;

	for (i = 0; i < size; i++) {
		a = qce->isobuf[i];
		b = qce->isobuf[i+1];
		c = qce->isobuf[i+2];
		d = qce->isobuf[i+3];

		/* frame header and footer */
		if ((a == 0x80 || a == 0xC0) && c == 0 && d == 0) {
			if (b == 0x01 && qce->recording == 0) {
				//fprintf(stderr, "New frame at %lu.\n", i);
				i += 3;
				qce->pos = 0;
				qce->recording = 1;
			} else if (b == 0x02 && qce->recording == 1) {
				i += 4;
				qce->framecount++;
				qce->resume = size - i;
				bcopy(qce->isobuf + i, qce->isobuf, qce->resume);
				//fprintf(stderr, "EOF at %lu, saving %lu bytes for later.\n", i, qce->resume);
				//fprintf(stderr, "--> Next bytes : %02X %02X %02X %02X\n", qce->isobuf[i], qce->isobuf[i+1], qce->isobuf[i+2], qce->isobuf[i+3]);
				qce->recording = 0;
				return (1);
			}

		/* picture data */
		} else {
			if (a == 0x02 && b == 0x00) {
				csize = (c << 8) + d;
				//fprintf(stderr, " + chunk of %u (%lu bytes left)\n", csize, size - i);

				/* chunk is too big, we keep it for the next parse */
				if (csize > size - i) {
					//fprintf(stderr, "chunk is too big, keeping %lu bytes\n", size - i);
					qce->resume = size - i;
					bcopy(qce->isobuf + i, qce->isobuf, qce->resume);
					return (0);
				}

				if (qce->recording)
					save_chunk(qce, qce->isobuf + i + 4, csize);
				i += 3 + csize;
			} else
				fprintf(stderr, "I lost sync (while recording) ! byte is %02X\n", a);
		}

	}

	return (0);
}


void
quickcam_capture_start(struct quickcam *qce)
{
	hdcs_set1(qce, HDSC_CONTROL, 0x04);
	stv_set_byte(qce, STV_STREAMING, 0x01);
}


void
quickcam_capture_stop(struct quickcam *qce)
{
	stv_set_byte(qce, STV_STREAMING, 0x00);
	hdcs_set1(qce, HDSC_CONTROL, 0x00);
}


void
quickcam_next_frame(struct quickcam *qce)
{
	unsigned long i;

	//fprintf(stderr, "quickcam_next_frame\n");

	for (;;) {
		//fprintf(stderr, "resuming %lu bytes.\n", qce->resume);
		i = read(qce->iso_ep, qce->isobuf + qce->resume, 65535);
		if (parse_iso(qce, i + qce->resume))
			break;
	}

}

