/* $Id: sdlviewer.c,v 1.3 2007/02/21 18:12:13 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>

#include "openqce.h"


void
usage(void)
{
	extern char *__progname;

	fprintf(stderr, "Usage: %s [-h] [-d devname] [-s vidsize] [-f filter]\n", __progname);
	fprintf(stderr, "\nCurrently supported camera resolutions (-s) :\n");
	fprintf(stderr, "        -s 0    376x312: full array with borders\n");
	fprintf(stderr, "        -s 1    360x296: maximum picture\n");
	fprintf(stderr, "        -s 2    352x288: standard CIF size (default)\n");
	fprintf(stderr, "\nCurrently supported filters (-f) :\n");
	fprintf(stderr, "        -f 1    Greyscale Raw Bayer\n");
	fprintf(stderr, "        -f 2    Colored Raw Bayer\n");
	fprintf(stderr, "        -f 3    Fast 2x2, (cut res. by 2)\n");
	fprintf(stderr, "        -f 4    Full 2x2 (default)\n");
	exit(-1);
}


/* 
 * Use the force Luke.
 */
int main(int ac, char **av)
{
	struct quickcam *qce;
	byte *t;
	int ch, running = 1;
	SDL_Surface *pic;
	SDL_Surface *screen;
	SDL_Rect f;
	SDL_Event event;

	qce = quickcam_create();

	while ((ch = getopt(ac, av, "f:s:d:h")) != -1) {
		switch (ch) {
			case 'f':
				qce->format = strtol(optarg, NULL, 10);
				if (qce->format > 4)
					qce->format = DEFAULT_FORMAT;
				break;
			case 'd':
				strlcpy(qce->devname, optarg, 11);
				break;
			case 's':
				qce->mode = strtol(optarg, NULL, 10);
				if (qce->mode > 2)
					qce->mode = DEFAULT_VSIZE;
				break;
			case 'h':
			default:
				usage();
				break;
		}
	}
	ac -= optind;
	av += optind;

	quickcam_init(qce);

	SDL_Init(SDL_INIT_VIDEO);
	SDL_WM_SetCaption("OpenQCE SDL Viewer", "openqce");
	atexit(SDL_Quit);
	screen = SDL_SetVideoMode(qce->width, qce->height, 0, SDL_HWSURFACE|SDL_DOUBLEBUF);
	pic = SDL_CreateRGBSurface(SDL_HWSURFACE, qce->width, qce->height, 24, 0x00FF0000, 0x0000FF00, 0x000000FF, 0x00000000);
	t = pic->pixels;
	f.x = 0;
	f.y = 0;
	f.w = qce->width;
	f.h = qce->height;

	quickcam_capture_start(qce);

	for (; running; ) {
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_KEYDOWN:
				case SDL_QUIT:
					running = 0;
			}
		}
	 	quickcam_next_frame(qce);
		switch (qce->format) {
			case 0:
			default:
			case 1:
				filter_render_grey(qce);
				break;
			case 2:
				filter_render_rgb(qce);
				break;
			case 3:
				filter_render_2x2(qce);
				break;
			case 4:
				filter_render_full2x2(qce);
				break;
		}
		pic->pixels = qce->rgbframe;
		SDL_BlitSurface(pic, &f, screen, &f);
		SDL_Flip(screen);
	}

	quickcam_capture_stop(qce);

	quickcam_shutdown(qce);

	exit(0);
}

